import base64
import os

from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models

path = "darknet/data"


class AnnotationImageModel(models.Model):
    image = models.FileField(upload_to=path)
    annotate = models.FileField(upload_to=path)

    def __str__(self):
        return str(image)


class DontFakeMe(models.Model):
    image = models.ImageField(upload_to='images')

    def get_cover_base64(self, format='png'):
        return image_as_base64(self.image.path, format)

    def __str__(self):
        return self.image


def image_as_base64(image_file, format='png'):
    """
    :param image_file: complete path of image
    :param format: supports png, jpg
    """
    print(image_file)
    if not os.path.isfile(image_file):
        return None

    encoded_string = ''
    with open(image_file, 'rb') as img_f:
        encoded_string = base64.b64encode(img_f.read())
    return 'data:image/%s;base64,%s' % (format, encoded_string.decode('utf-8'))


class YoloClassModel(models.Model):
    name = models.CharField(max_length=30, unique=True)
    total = models.IntegerField(default=0)

    def __str__(self):
        return str(self.name)


class ImageDetectionModel(models.Model):
    image = models.ImageField(upload_to='uploaded')


class ConfigurationModel(models.Model):
    train_percentage = models.IntegerField(default=80, validators=[MaxValueValidator(100), MinValueValidator(1)])
    test_percentage = models.IntegerField(default=20)



from django.urls import path
from .views import upload, home, stat, yolo_class, detection_image, model, model_settings


urlpatterns = [
    path('', home, name="home"),
    path('upload/', upload, name="upload"),
    path('stat', stat, name="stat"),
    path('stat/class', yolo_class, name="yolo_class"),
    path('detection_image', detection_image, name="detection_image"),
    path('model', model, name="model"),
    path('settings', model_settings, name="model_settings"),
]


# Generated by Django 3.0.3 on 2020-03-12 08:54

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0007_auto_20200304_1915'),
    ]

    operations = [
        migrations.CreateModel(
            name='ConfigurationModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('train_percentage', models.IntegerField(default=80, validators=[django.core.validators.MaxValueValidator(100), django.core.validators.MinValueValidator(1)])),
                ('test_percentage', models.IntegerField(default=20)),
            ],
        ),
        migrations.CreateModel(
            name='ImageDetectionModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to='uploaded')),
                ('stat', models.TextField()),
            ],
        ),
        migrations.RenameModel(
            old_name='YoloClass',
            new_name='YoloClassModel',
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-02 17:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0004_auto_20200302_1703'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dontfakeme',
            name='image',
            field=models.ImageField(upload_to='static/images/'),
        ),
    ]



import json

from django.conf import settings
from django.http import JsonResponse
from django.shortcuts import render

from .models import AnnotationImageModel, YoloClassModel, ImageDetectionModel

file_supported = ['png', 'PNG', 'jpg', 'jpeg', 'JPG', 'JPEG']


def upload(request):
    context = {}
    table = YoloClassModel.objects.all().values('name')
    context['class_table'] = list(table)
    annotate = request.POST.get('annotate-files', '')
    print(annotate)
    if request.method == 'POST':
        print(request.FILES)
        context['msg_class'] = 'is-primary'
        print(request.POST.keys())
        for i in request.POST.keys():
            print(request.POST.get(i))
        for field in request.FILES.keys():
            print(field)
            for formfile in request.FILES.getlist(field):
                # check if file is supported
                print(formfile)
                if str(formfile.name).split(".")[1] not in file_supported:
                    context['msg_class'] = 'is-danger'
                    context['message'] = 'File extension not supported!'
                    return render(request, 'upload.html', context=context)

                img = AnnotationImageModel(image=formfile)
                img.save()
                # create annotation file
                annotate = request.POST.get('annotate-files', '')
                annotate_path = settings.MEDIA_ROOT + '/' + str(img.image).replace(str(img.image).split('.')[1], 'txt')
                f = open(annotate_path, 'w+')
                f.write(json.loads(annotate))

                img.annotate = annotate_path
                img.save()

                f.close()

        context['message'] = 'File uploaded!'
    return render(request, 'upload.html', context=context)


def home(request):
    print(request.META.get("REMOTE_ADDR"))
    return render(request, 'home.html')


def analysis(request):
    return render(request, 'analysis.html')


def stat(request):
    context = {}
    context['card'] =[{
        'name': 'CLASS',
        'value': YoloClassModel.objects.count(),
    }, {
        'name': 'Images',
        'value': AnnotationImageModel.objects.count(),
    }, {
        'name': 'Response Time',
        'value': str(2.5) + 'ms',
    }]
    return render(request, 'stat.html', context=context)


def yolo_class(request):
    context = {}
    table = YoloClassModel.objects.all().values()
    context['class_table'] = table
    print(table)
    return render(request, 'class.html', context=context)


def detection_image(request):
    if request.FILES:
        image = request.FILES['image']
        model = ImageDetectionModel(image=image)
        return JsonResponse({
            "status": True,

        })
    return JsonResponse({
        "status": False,
    })


def model(request):
    context = {}
    context['log'] = read_file(settings.MEDIA_ROOT + '/darknet/log.txt')
    return render(request, "model.html", context=context)


def model_settings(request):
    context = {}
    main_path = settings.MEDIA_ROOT + '/config/'
    # create class file
    f = open(main_path + 'classes.name', 'w+')
    class_model = YoloClassModel.objects.all()
    total_classes = YoloClassModel.objects.count()
    for c in class_model:
        f.write(c.name + "\n")
    f.close()
    context['classes'] = read_file(main_path + 'classes.name')
    context['total_class'] = total_classes
    # configure data file
    f = open(main_path + 'darknet.data', 'w')
    f.write('classes = ' + str(total_classes) + '\n')
    f.write('train = ' + main_path + 'train.txt\n')
    f.write('valid = ' + main_path + 'test.txt\n')
    f.write('names = ' + main_path + 'classes.names\n')
    f.write('backup = ' + settings.MEDIA_ROOT + '/weights/\n')
    f.close()

    # yolo configuration file
    context['image_width'] = ''
    context['image_height'] = ''
    with open(settings.MEDIA_ROOT + '/darknet/cfg/yolov3.cfg', 'r') as fp:
        for line in fp:
            print(line)
            context['image_width'] = line.replace("width=", "") if line.__contains__("width=") else context['image_width']
            context['image_height'] = line.replace("height=", "") if line.__contains__("height=") else context['image_height']
    return render(request, "settings.html", context=context)


def train(request):
    return JsonResponse({
        "status": True
    })



# Generated by Django 3.0.3 on 2020-03-04 19:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0005_auto_20200302_1710'),
    ]

    operations = [
        migrations.CreateModel(
            name='YoloClass',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('total', models.IntegerField(default=0)),
            ],
        ),
        migrations.AlterField(
            model_name='dontfakeme',
            name='image',
            field=models.ImageField(upload_to='images'),
        ),
    ]



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()

version = '2020.1.0'

install_requires = [
    "asgiref==3.2.5",
    "autocorrect==0.4.4",
    "Django>=2.0.7",
    "opencv-python==4.2.0.32",
    "pytz==2019.3",
    "sqlparse==0.3.1"
]

setup(name='djanyolo',
      version=version,
      description="Manage content for YOLO model.",
      long_description=README + '\n\n' + NEWS,
      classifiers=[
      ],
      keywords='yolo, djanyolo, django',
      author='RoboMx Team',
      author_email='ask@robomx.tech',
      url='https://robomx.com',
      license='MIT',
      packages=find_packages('src'),
      package_dir={'': 'src'}, include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      entry_points={
          'console_scripts':
              ['djanyolo=djanyolo:main']
      }
      )



# Generated by Django 3.0.3 on 2020-02-27 15:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0002_auto_20200225_1930'),
    ]

    operations = [
        migrations.CreateModel(
            name='AnnotationImageModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.FileField(upload_to='media/darknet/data/custom')),
                ('annotate', models.FileField(upload_to='media/darknet/data/custom')),
            ],
        ),
        migrations.RemoveField(
            model_name='objectclassmodel',
            name='user',
        ),
        migrations.DeleteModel(
            name='AnnotationClass',
        ),
        migrations.DeleteModel(
            name='ObjectClassModel',
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-02 17:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0003_auto_20200227_1539'),
    ]

    operations = [
        migrations.CreateModel(
            name='DontFakeMe',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.FileField(upload_to='images')),
            ],
        ),
        migrations.AlterField(
            model_name='annotationimagemodel',
            name='annotate',
            field=models.FileField(upload_to='darknet/data/custom'),
        ),
        migrations.AlterField(
            model_name='annotationimagemodel',
            name='image',
            field=models.FileField(upload_to='darknet/data/custom'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-04 19:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0006_auto_20200304_1910'),
    ]

    operations = [
        migrations.AlterField(
            model_name='yoloclass',
            name='name',
            field=models.CharField(max_length=30, unique=True),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-25 19:30

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('main', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='annotationclass',
            name='user',
            field=models.ForeignKey(default='', on_delete=django.db.models.deletion.DO_NOTHING, to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='objectclassmodel',
            name='user',
            field=models.ForeignKey(default='', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
    ]



from django.contrib import admin
from .models import AnnotationImageModel, YoloClassModel


@admin.register(AnnotationImageModel)
class ObjectClassAdmin(admin.ModelAdmin):
    pass


@admin.register(YoloClassModel)
class YoloClassAdmin(admin.ModelAdmin):
    pass